@@ -2,7 +2,7 @@
 **  Copyright (c) 2007-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010 The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011 The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef lint
@@ -182,9 +182,11 @@ dkim_canon_write(DKIM_CANON *canon, u_char *buf, size_t buflen)
 {
 	assert(canon != NULL);
 
-	if (canon->canon_remain != (off_t) -1)
+	if (canon->canon_remain != (ssize_t) -1)
 		buflen = MIN(buflen, canon->canon_remain);
 
+	canon->canon_wrote += buflen;
+
 	if (buf == NULL || buflen == 0)
 		return;
 
@@ -238,8 +240,7 @@ dkim_canon_write(DKIM_CANON *canon, u_char *buf, size_t buflen)
 #endif /* USE_GNUTLS */
 	}
 
-	canon->canon_wrote += buflen;
-	if (canon->canon_remain != (off_t) -1)
+	if (canon->canon_remain != (ssize_t) -1)
 		canon->canon_remain -= buflen;
 }
 
@@ -838,7 +839,7 @@ dkim_canon_cleanup(DKIM *dkim)
 DKIM_STAT
 dkim_add_canon(DKIM *dkim, _Bool hdr, dkim_canon_t canon, int hashtype,
                u_char *hdrlist, struct dkim_header *sighdr,
-               off_t length, DKIM_CANON **cout)
+               ssize_t length, DKIM_CANON **cout)
 {
 	DKIM_CANON *cur;
 	DKIM_CANON *new;
@@ -890,8 +891,8 @@ dkim_add_canon(DKIM *dkim, _Bool hdr, dkim_canon_t canon, int hashtype,
 	new->canon_wrote = 0;
 	if (hdr)
 	{
-		new->canon_length = (off_t) -1;
-		new->canon_remain = (off_t) -1;
+		new->canon_length = (ssize_t) -1;
+		new->canon_remain = (ssize_t) -1;
 	}
 	else
 	{
@@ -1617,7 +1618,7 @@ dkim_canon_minbody(DKIM *dkim)
 			continue;
 
 		/* if this one wants the whole message, short-circuit */
-		if (cur->canon_remain == (off_t) -1)
+		if (cur->canon_remain == (ssize_t) -1)
 			return ULONG_MAX;
 
 		/* compare to current minimum */
@@ -1657,8 +1658,7 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 
 	dkim->dkim_bodylen += buflen;
 
-	fixcrlf = (dkim->dkim_libhandle->dkiml_flags & DKIM_LIBFLAGS_FIXCRLF) != 0 &&
-	          (dkim->dkim_mode == DKIM_MODE_SIGN);
+	fixcrlf = (dkim->dkim_libhandle->dkiml_flags & DKIM_LIBFLAGS_FIXCRLF);
 
 	for (cur = dkim->dkim_canonhead; cur != NULL; cur = cur->canon_next)
 	{
@@ -1666,10 +1666,6 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 		if (cur->canon_done || cur->canon_hdr)
 			continue;
 
-		/* short-circuit completed canonicalizations */
-		if (cur->canon_remain == 0)
-			continue;
-
 		start = buf;
 		plen = buflen;
 
@@ -1746,6 +1742,7 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 							dkim_canon_flushblanks(cur);
 						cur->canon_blankline = FALSE;
 					}
+
 					wlen++;
 				}
 
