@@ -2,11 +2,11 @@
 **  Copyright (c) 2005-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef lint
-static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.14.2.1 2010/07/08 23:50:10 cm-msk Exp $";
+static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.18.10.1 2010/10/27 21:43:08 cm-msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -25,20 +25,12 @@ static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.14.2.1 2010/07/08 23:50
 
 #include "build-config.h"
 
-/* libar includes */
-#if USE_ARLIB
-# include "ar.h"
-#endif /* USE_ARLIB */
-
 /* libopendkim includes */
 #include "dkim-internal.h"
 #include "dkim-types.h"
 #include "dkim-keys.h"
 #include "dkim-cache.h"
 #include "dkim-test.h"
-#if USE_UNBOUND
-# include "dkim-ub.h"
-#endif /* USE_UNBOUND */
 #include "util.h"
 #include "dkim-strl.h"
 
@@ -52,6 +44,9 @@ extern void dkim_error __P((DKIM *, const char *, ...));
 #else /* __RES && __RES >= 19940415 */
 # define RES_UNC_T		unsigned char *
 #endif /* __RES && __RES >= 19940415 */
+#ifndef T_RRSIG
+# define T_RRSIG		46
+#endif /* ! T_RRSIG */
 
 /*
 **  DKIM_GET_KEY_DNS -- retrieve a DKIM key from DNS
@@ -70,37 +65,30 @@ DKIM_STAT
 dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 {
 #ifdef QUERY_CACHE
-	bool cached = FALSE;
+	_Bool cached = FALSE;
 	uint32_t ttl = 0;
 #endif /* QUERY_CACHE */
 	int status;
 	int qdcount;
 	int ancount;
-#if USE_ARLIB
 	int error;
-#endif /* USE_ARLIB */
+	int dnssec = DKIM_DNSSEC_UNKNOWN;
 	int c;
 	int n = 0;
+	int rdlength;
 	int type = -1;
 	int class = -1;
 	size_t anslen;
-#if USE_UNBOUND
-	struct dkim_unbound_cb_data cb_data;
-#endif /* USE_UNBOUND */
-#if USE_ARLIB
-	AR_LIB ar;
-	AR_QUERY q;
-#endif /* USE_ARLIB */
+	void *q;
 	DKIM_LIB *lib;
+	unsigned char *txtfound = NULL;
 	unsigned char *p;
 	unsigned char *cp;
 	unsigned char *eom;
 	unsigned char *eob;
-	char qname[DKIM_MAXHOSTNAMELEN + 1];
+	unsigned char qname[DKIM_MAXHOSTNAMELEN + 1];
 	unsigned char ansbuf[MAXPACKET];
-#if USE_ARLIB
 	struct timeval timeout;
-#endif /* USE_ARLIB */
 	HEADER hdr;
 
 	assert(dkim != NULL);
@@ -111,8 +99,8 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 
 	lib = dkim->dkim_libhandle;
 
-	n = snprintf(qname, sizeof qname - 1, "%s.%s.%s", sig->sig_selector,
-	             DKIM_DNSKEYNAME, sig->sig_domain);
+	n = snprintf((char *) qname, sizeof qname - 1, "%s.%s.%s",
+	             sig->sig_selector, DKIM_DNSKEYNAME, sig->sig_domain);
 	if (n == -1 || n > sizeof qname - 1)
 	{
 		dkim_error(dkim, "key query name too large");
@@ -143,181 +131,88 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	anslen = dkim_test_dns_get(dkim, ansbuf, sizeof ansbuf);
 	if (anslen == -1)
 	{
-#if USE_UNBOUND
-		status = dkim_unbound_queue(dkim, qname, T_TXT,
-		                            buf, buflen, &cb_data);
-
-		if (status != 0)
-		{
-			dkim_error(dkim, "error queueing DNS request for `%s'",
-			           qname);
-			return DKIM_STAT_KEYFAIL;
-		}
-
-		status = dkim_unbound_wait(dkim, &cb_data);
-
-		switch (status)
-		{
-		  case 1:
-			sig->sig_dnssec_key = cb_data.ubd_result;
-			break;
-
-		  case 0:
-			dkim_error(dkim, "timeout DNS query for `%s'", qname);
-			return DKIM_STAT_KEYFAIL;
-
-		  case -1:
-			dkim_error(dkim, "error processing DNS query for `%s'",
-			           qname);
-			if (cb_data.ubd_stat != DKIM_STAT_OK)
-				return cb_data.ubd_stat;
-			else
-				return DKIM_STAT_INTERNAL;
-		}
-	}
-#else /* USE_UNBOUND */
-# if USE_ARLIB
-#  ifdef _FFR_DNSUPGRADE
-		for (c = 0; c < 2; c++)
-		{
-			switch (c)
-			{
-			  case 0:
-				ar = dkim->dkim_libhandle->dkiml_arlib;
-				break;
-
-			  case 1:
-				ar = dkim->dkim_libhandle->dkiml_arlibtcp;
-				break;
-			}
-
-			timeout.tv_sec = dkim->dkim_timeout;
-			timeout.tv_usec = 0;
-
-			q = ar_addquery(ar, qname, C_IN, T_TXT, MAXCNAMEDEPTH,
-			                ansbuf, sizeof ansbuf, &error,
-			                dkim->dkim_timeout == 0 ? NULL
-			                                        : &timeout);
-			if (q == NULL)
-			{
-				dkim_error(dkim,
-				           "ar_addquery() for `%s' failed",
-				           qname);
-				return DKIM_STAT_INTERNAL;
-			}
-
-			if (lib->dkiml_dns_callback == NULL)
-			{
-				status = ar_waitreply(ar, q, &anslen, NULL);
-			}
-			else
-			{
-				for (;;)
-				{
-					timeout.tv_sec = lib->dkiml_callback_int;
-					timeout.tv_usec = 0;
-
-					status = ar_waitreply(ar, q, &anslen,
-					                      &timeout);
-
-					if (status != AR_STAT_NOREPLY)
-						break;
-
-					lib->dkiml_dns_callback(dkim->dkim_user_context);
-				}
-			}
-
-			(void) ar_cancelquery(ar, q);
-
-			/* see if the UDP reply was truncated */
-			if (c == 0 && status == AR_STAT_SUCCESS)
-			{
-				memcpy(&hdr, ansbuf, sizeof hdr);
-				if (dkim_check_dns_reply(ansbuf, anslen,
-			                         C_IN, T_TXT) == 1)
-					continue;
-			}
-
-			break;
-		}
-#  else /* _FFR_DNSUPGRADE */
-		ar = dkim->dkim_libhandle->dkiml_arlib;
+		anslen = sizeof ansbuf;
 
 		timeout.tv_sec = dkim->dkim_timeout;
 		timeout.tv_usec = 0;
 
-		q = ar_addquery(ar, qname, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
-		                sizeof ansbuf, &error,
-		                dkim->dkim_timeout == 0 ? NULL : &timeout);
-		if (q == NULL)
-		{
-			dkim_error(dkim, "ar_addquery() for `%s' failed",
-			           qname);
-			return DKIM_STAT_INTERNAL;
-		}
+		status = lib->dkiml_dns_start(lib->dkiml_dns_service, T_TXT,
+		                              qname, ansbuf, anslen, &q);
 
+		if (status != 0)
+		{
+			dkim_error(dkim, "'%s' query failed", qname);
+			return DKIM_STAT_KEYFAIL;
+		}
+	
 		if (lib->dkiml_dns_callback == NULL)
 		{
-			status = ar_waitreply(ar, q, &anslen, NULL);
+			timeout.tv_sec = dkim->dkim_timeout;
+			timeout.tv_usec = 0;
+
+			status = lib->dkiml_dns_waitreply(lib->dkiml_dns_service,
+			                                  q,
+			                                  dkim->dkim_timeout == 0 ? NULL
+			                                                          : &timeout,
+			                                  &anslen, &error,
+			                                  &dnssec);
 		}
 		else
 		{
+			struct timeval master;
+			struct timeval next;
+			struct timeval *wt;
+
+			(void) gettimeofday(&master, NULL);
+			master.tv_sec += dkim->dkim_timeout;
+
 			for (;;)
 			{
-				timeout.tv_sec = lib->dkiml_callback_int;
-				timeout.tv_usec = 0;
+				(void) gettimeofday(&next, NULL);
+				next.tv_sec += lib->dkiml_callback_int;
 
-				status = ar_waitreply(ar, q, &anslen,
-				                      &timeout);
+				dkim_min_timeval(&master, &next,
+				                 &timeout, &wt);
 
-				if (status != AR_STAT_NOREPLY)
+				status = lib->dkiml_dns_waitreply(lib->dkiml_dns_service,
+				                                  q,
+				                                  dkim->dkim_timeout == 0 ? NULL
+				                                                          : &timeout,
+				                                  &anslen,
+				                                  &error,
+				                                  &dnssec);
+
+				if (wt == &next)
+				{
+					if (status == DKIM_DNS_NOREPLY ||
+					    status == DKIM_DNS_EXPIRED)
+						lib->dkiml_dns_callback(dkim->dkim_user_context);
+					else
+						break;
+				}
+				else
+				{
 					break;
-
-				lib->dkiml_dns_callback(dkim->dkim_user_context);
+				}
 			}
 		}
 
-		(void) ar_cancelquery(ar, q);
-#  endif /* _FFR_DNSUPGRADE */
-# else /* USE_ARLIB */
-		status = res_query(qname, C_IN, T_TXT, ansbuf, sizeof ansbuf);
-# endif /* USE_ARLIB */
-
-# if USE_ARLIB
-		if (status == AR_STAT_ERROR || status == AR_STAT_EXPIRED)
+		if (status == DKIM_DNS_EXPIRED)
 		{
-			dkim_error(dkim, "ar_waitreply(): `%s' %s", qname,
-			           status == AR_STAT_ERROR ? "error"
-			                                   : "expired");
+			(void) lib->dkiml_dns_cancel(lib->dkiml_dns_service, q);
+			dkim_error(dkim, "'%s' query timed out", qname);
 			return DKIM_STAT_KEYFAIL;
 		}
-# else /* USE_ARLIB */
-		/*
-		**  A -1 return from res_query could mean a bunch of things,
-		**  not just NXDOMAIN.  You can use h_errno to determine what
-		**  -1 means.  This is poorly documented.
-		*/
-
-		if (status == -1)
+		else if (status == DKIM_DNS_ERROR)
 		{
-			dkim_error(dkim, "res_query(): `%s' %s",
-			           qname, hstrerror(h_errno));
-
-			switch (h_errno)
-			{
-			  case HOST_NOT_FOUND:
-			  case NO_DATA:
-				return DKIM_STAT_NOKEY;
-
-			  case TRY_AGAIN:
-			  case NO_RECOVERY:
-			  default:
-				return DKIM_STAT_KEYFAIL;
-			}
+			(void) lib->dkiml_dns_cancel(lib->dkiml_dns_service, q);
+			dkim_error(dkim, "'%s' query failed", qname);
+			return DKIM_STAT_KEYFAIL;
 		}
 
-		anslen = status;
-# endif /* USE_ARLIB */
+		(void) lib->dkiml_dns_cancel(lib->dkiml_dns_service, q);
+
+		sig->sig_dnssec_key = dnssec;
 	}
 
 	/* set up pointers */
@@ -331,12 +226,12 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	     qdcount--)
 	{
 		/* copy it first */
-		(void) dn_expand((unsigned char *) &ansbuf, eom, cp, qname,
-		                 sizeof qname);
-
+		(void) dn_expand((unsigned char *) &ansbuf, eom, cp,
+		                 (char *) qname, sizeof qname);
+ 
 		if ((n = dn_skipname(cp, eom)) < 0)
 		{
-			dkim_error(dkim, "`%s' reply corrupt", qname);
+			dkim_error(dkim, "'%s' reply corrupt", qname);
 			return DKIM_STAT_KEYFAIL;
 		}
 		cp += n;
@@ -344,7 +239,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		/* extract the type and class */
 		if (cp + INT16SZ + INT16SZ > eom)
 		{
-			dkim_error(dkim, "`%s' reply corrupt", qname);
+			dkim_error(dkim, "'%s' reply corrupt", qname);
 			return DKIM_STAT_KEYFAIL;
 		}
 		GETSHORT(type, cp);
@@ -353,21 +248,22 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 
 	if (type != T_TXT || class != C_IN)
 	{
-		dkim_error(dkim, "`%s' unexpected reply type/class", qname);
+		dkim_error(dkim, "'%s' unexpected reply class/type (%d/%d)",
+		           qname, class, type);
 		return DKIM_STAT_KEYFAIL;
 	}
 
 	/* if NXDOMAIN, return DKIM_STAT_NOKEY */
 	if (hdr.rcode == NXDOMAIN)
 	{
-		dkim_error(dkim, "`%s' record not found", qname);
+		dkim_error(dkim, "'%s' record not found", qname);
 		return DKIM_STAT_NOKEY;
 	}
 
 	/* if truncated, we can't do it */
 	if (dkim_check_dns_reply(ansbuf, anslen, C_IN, T_TXT) == 1)
 	{
-		dkim_error(dkim, "`%s' reply truncated", qname);
+		dkim_error(dkim, "'%s' reply truncated", qname);
 		return DKIM_STAT_KEYFAIL;
 	}
 
@@ -386,81 +282,81 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		if ((n = dn_expand((unsigned char *) &ansbuf, eom, cp,
 		                   (RES_UNC_T) qname, sizeof qname)) < 0)
 		{
-			dkim_error(dkim, "`%s' reply corrupt", qname);
+			dkim_error(dkim, "'%s' reply corrupt", qname);
 			return DKIM_STAT_KEYFAIL;
 		}
 		/* ...and move past it */
 		cp += n;
 
 		/* extract the type and class */
-		if (cp + INT16SZ + INT16SZ > eom)
+		if (cp + INT16SZ + INT16SZ + INT32SZ + INT16SZ > eom)
 		{
-			dkim_error(dkim, "`%s' reply corrupt", qname);
+			dkim_error(dkim, "'%s' reply corrupt", qname);
 			return DKIM_STAT_KEYFAIL;
 		}
 
-		GETSHORT(type, cp);
-		GETSHORT(class, cp);
-
-# ifdef QUERY_CACHE
+		GETSHORT(type, cp);			/* TYPE */
+		GETSHORT(class, cp);			/* CLASS */
+#ifdef QUERY_CACHE
 		/* get the TTL */
-		GETLONG(ttl, cp);
-# else /* QUERY_CACHE */
+		GETLONG(ttl, cp);			/* TTL */
+#else /* QUERY_CACHE */
 		/* skip the TTL */
-		cp += INT32SZ;
-# endif /* QUERY_CACHE */
+		cp += INT32SZ;				/* TTL */
+#endif /* QUERY_CACHE */
+		GETSHORT(n, cp);			/* RDLENGTH */
 
 		/* skip CNAME if found; assume it was resolved */
 		if (type == T_CNAME)
 		{
-			char chost[DKIM_MAXHOSTNAMELEN + 1];
-
-			n = dn_expand((u_char *) &ansbuf, eom, cp,
-			              chost, DKIM_MAXHOSTNAMELEN);
+			cp += n;
+			continue;
+		}
+		else if (type == T_RRSIG)
+		{
 			cp += n;
 			continue;
 		}
 		else if (type != T_TXT)
 		{
-			dkim_error(dkim, "`%s' reply was unexpected type %d",
+			dkim_error(dkim, "'%s' reply was unexpected type %d",
 			           qname, type);
 			return DKIM_STAT_KEYFAIL;
 		}
 
-		if (ancount > 0)
+		if (txtfound != NULL)
 		{
-			dkim_error(dkim, "multiple DNS replies for `%s'",
+			dkim_error(dkim, "multiple DNS replies for '%s'",
 			           qname);
 			return DKIM_STAT_MULTIDNSREPLY;
 		}
 
-		/* found a record we can use; break */
-		break;
+		/* remember where this one started */
+		txtfound = cp;
+		rdlength = n;
+
+		/* move forward for now */
+		cp += n;
 	}
 
 	/* if ancount went below 0, there were no good records */
-	if (ancount < 0)
+	if (txtfound == NULL)
 	{
-		dkim_error(dkim, "`%s' reply was unresolved CNAME", qname);
+		dkim_error(dkim, "'%s' reply was unresolved CNAME", qname);
 		return DKIM_STAT_KEYFAIL;
 	}
 
-	/* get payload length */
-	if (cp + INT16SZ > eom)
-	{
-		dkim_error(dkim, "`%s' reply corrupt", qname);
-		return DKIM_STAT_KEYFAIL;
-	}
-	GETSHORT(n, cp);
+	/* come back to the one we found */
+	cp = txtfound;
 
 	/*
 	**  XXX -- maybe deal with a partial reply rather than require
 	**  	   it all
 	*/
 
-	if (cp + n > eom)
+	if (cp + rdlength > eom)
 	{
-		dkim_error(dkim, "`%s' reply corrupt", qname);
+		dkim_error(dkim, "'%s' reply corrupt", qname);
 		return DKIM_STAT_SYNTAX;
 	}
 
@@ -468,18 +364,17 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	memset(buf, '\0', buflen);
 	p = buf;
 	eob = buf + buflen - 1;
-	while (n > 0 && p < eob)
+	while (rdlength > 0 && p < eob)
 	{
 		c = *cp++;
-		n--;
+		rdlength--;
 		while (c > 0 && p < eob)
 		{
 			*p++ = *cp++;
 			c--;
-			n--;
+			rdlength--;
 		}
 	}
-#endif /* USE_UNBOUND */
 
 #ifdef QUERY_CACHE
 	if (!cached && buf[0] != '\0' &&
@@ -527,7 +422,7 @@ dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	FILE *f;
 	u_char *p;
 	u_char *p2;
-	char *path;
+	u_char *path;
 	char name[DKIM_MAXHOSTNAMELEN + 1];
 
 	assert(dkim != NULL);
@@ -543,7 +438,7 @@ dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		return DKIM_STAT_KEYFAIL;
 	}
 
-	f = fopen(path, "r");
+	f = fopen((char *) path, "r");
 	if (f == NULL)
 	{
 		dkim_error(dkim, "%s: fopen(): %s", path, strerror(errno));
@@ -560,7 +455,7 @@ dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	}
 
 	memset(buf, '\0', buflen);
-	while (fgets(buf, buflen, f) != NULL)
+	while (fgets((char *) buf, buflen, f) != NULL)
 	{
 		if (buf[0] == '#')
 			continue;
@@ -585,9 +480,9 @@ dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			}
 		}
 
-		if (strcasecmp(name, buf) == 0 && p2 != NULL)
+		if (strcasecmp((char *) name, (char *) buf) == 0 && p2 != NULL)
 		{
-			strlcpy(buf, p2, buflen);
+			strlcpy((char *) buf, (char *) p2, buflen);
 			fclose(f);
 			return DKIM_STAT_OK;
 		}
