@@ -56,6 +56,23 @@ static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.23 2010/10/28 02:41:2
 #include "dkim.h"
 #include "dkim-internal.h"
 
+#ifdef _FFR_XTAGS
+/* struct dkim_xtag -- signature extension tag */
+struct dkim_xtag
+{
+	const char *		xt_tag;
+	const char *		xt_value;
+	struct dkim_xtag *	xt_next;
+};
+#endif /* _FFR_XTAGS */
+
+/* struct dkim_queryinfo -- DNS query information */
+struct dkim_queryinfo
+{
+	int			dq_type;
+	const char		dq_name[DKIM_MAXHOSTNAMELEN + 1];
+};
+
 /* struct dkim_pstate -- policy query state */
 struct dkim_pstate
 {
@@ -184,9 +201,9 @@ struct dkim_canon
 	u_int			canon_blanks;
 	size_t			canon_hashbuflen;
 	size_t			canon_hashbufsize;
-	off_t			canon_remain;
-	off_t			canon_wrote;
-	off_t			canon_length;
+	ssize_t			canon_remain;
+	ssize_t			canon_wrote;
+	ssize_t			canon_length;
 	dkim_canon_t		canon_canon;
 	u_char *		canon_hashbuf;
 	u_char *		canon_hdrlist;
@@ -259,6 +276,7 @@ struct dkim
 	int			dkim_state;
 	int			dkim_chunkstate;
 	int			dkim_chunksm;
+	int			dkim_chunkcrlf;
 	int			dkim_timeout;
 	int			dkim_presult;
 	int			dkim_hdrcnt;
@@ -288,8 +306,8 @@ struct dkim
 #ifdef _FFR_ATPS
 	_Bool			dkim_atps;
 #endif /* _FFR_ATPS */
-	off_t			dkim_bodylen;
-	off_t			dkim_signlen;
+	ssize_t			dkim_bodylen;
+	ssize_t			dkim_signlen;
 	const u_char *		dkim_id;
 	u_char *		dkim_domain;
 	u_char *		dkim_user;
@@ -310,6 +328,9 @@ struct dkim
 #ifdef _FFR_RESIGN
 	DKIM *			dkim_resign;
 #endif /* _FFR_RESIGN */
+#ifdef _FFR_XTAGS
+	struct dkim_xtag *	dkim_xtags;
+#endif /* _FFR_XTAGS */
 	struct dkim_siginfo **	dkim_siglist;
 	struct dkim_set *	dkim_sethead;
 	struct dkim_set *	dkim_settail;
