@@ -34,7 +34,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.36.2.1 2010/10/27 21:43:08 cm-msk
 **  	pp == patch number
 */
 
-#define	OPENDKIM_LIB_VERSION	0x02030200
+#define	OPENDKIM_LIB_VERSION	0x02040200
 
 #ifdef __STDC__
 # ifndef __P
@@ -47,6 +47,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.36.2.1 2010/10/27 21:43:08 cm-msk
 #endif /* __STDC__ */
 
 /* definitions */
+#define	DKIM_ATPSTAG		"atps"	/* special tag name */
 #define DKIM_HDRMARGIN		75	/* "standard" header margin */
 #define DKIM_MAXHEADER		4096	/* buffer for caching one header */
 #define	DKIM_MAXHOSTNAMELEN	256	/* max. FQDN we support */
@@ -154,11 +155,6 @@ typedef int DKIM_SIGERROR;
 #define	DKIM_DNS_NOREPLY	1		/* reply not available (yet) */
 #define	DKIM_DNS_EXPIRED	2		/* no reply, query expired */
 
-#define	DKIM_DNSSEC_UNKNOWN	(-1)		/* not checked */
-#define	DKIM_DNSSEC_BOGUS	0		/* forged */
-#define	DKIM_DNSSEC_INSECURE	1		/* unprotected */
-#define	DKIM_DNSSEC_SECURE	2		/* verified */
-
 /*
 **  DKIM_CANON -- canonicalization method
 */
@@ -289,16 +285,12 @@ typedef int dkim_opts_t;
 
 #define	DKIM_LIBFLAGS_DEFAULT		DKIM_LIBFLAGS_NONE
 
-#define DKIM_REP_DEFREJECT	1001
-#define DKIM_REP_ROOT		"al.dkim-reputation.org"
-
 /*
 **  DKIM_PFLAG -- policy flags
 */
 
 #define	DKIM_PFLAG_UNUSED1	0x01	/* no longer used */
 #define	DKIM_PFLAG_UNUSED2	0x02	/* no longer used */
-#define	DKIM_PFLAG_ATPS		0x04	/* atps */
 
 /*
 **  DKIM_DNSSEC -- results of DNSSEC queries
@@ -351,11 +343,19 @@ typedef struct dkim_siginfo DKIM_SIGINFO;
 #define DKIM_SIGFLAG_PASSED		0x04
 #define DKIM_SIGFLAG_TESTKEY		0x08
 #define DKIM_SIGFLAG_NOSUBDOMAIN	0x10
+#define DKIM_SIGFLAG_KEYLOADED		0x20
 
 #define DKIM_SIGBH_UNTESTED		(-1)
 #define DKIM_SIGBH_MATCH		0
 #define DKIM_SIGBH_MISMATCH		1
 
+/*
+**  DKIM_QUERYINFO -- information about a DNS query that is/may be needed
+*/
+
+struct dkim_queryinfo;
+typedef struct dkim_queryinfo DKIM_QUERYINFO;
+
 /*
 **  DKIM_PSTATE -- policy query state
 */
@@ -384,7 +384,7 @@ struct dkim_hdrdiff
 **  	None.
 **
 **  Return value:
-**  	A DKIM_STAT value.
+**  	A new DKIM library instance handle, or NULL on failure.
 */
 
 extern DKIM_LIB *dkim_init __P((void *(*mallocf)(void *closure, size_t nbytes),
@@ -435,7 +435,7 @@ extern DKIM *dkim_sign __P((DKIM_LIB *libhandle, const unsigned char *id,
                             dkim_canon_t hdr_canon_alg,
                             dkim_canon_t body_canon_alg,
                             dkim_alg_t sign_alg,
-                            off_t length, DKIM_STAT *statp));
+                            ssize_t length, DKIM_STAT *statp));
 
 /*
 **  DKIM_VERIFY -- make a new DKIM context for verifying
@@ -727,6 +727,23 @@ extern DKIM_STAT dkim_getsighdr_d __P((DKIM *dkim, size_t initial,
 
 extern _Bool dkim_sig_hdrsigned __P((DKIM_SIGINFO *sig, u_char *hdr));
 
+/*
+**  DKIM_SIG_GETQUERIES -- retrieve the queries needed to validate a signature
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
+**  	qi -- DKIM_QUERYINFO handle array (returned)
+**  	nqi -- number of entries in the "qi" array
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_sig_getqueries __P((DKIM *dkim, DKIM_SIGINFO *sig,
+                                          DKIM_QUERYINFO ***qi,
+                                          unsigned int *nqi));
+
 /*
 **  DKIM_SIG_GETDNSSEC -- retrieve DNSSEC results for a signature
 **
@@ -806,8 +823,8 @@ extern DKIM_STAT dkim_sig_getidentity __P((DKIM *dkim, DKIM_SIGINFO *sig,
 */
 
 extern DKIM_STAT dkim_sig_getcanonlen __P((DKIM *dkim, DKIM_SIGINFO *sig,
-                                           off_t *msglen, off_t *canonlen,
-                                           off_t *signlen));
+                                           ssize_t *msglen, ssize_t *canonlen,
+                                           ssize_t *signlen));
 
 /*
 **  DKIM_OPTIONS -- set/get options
@@ -846,11 +863,11 @@ extern unsigned int dkim_sig_getflags __P((DKIM_SIGINFO *sig));
 **  	sig -- DKIM_SIGINFO handle
 **
 **  Return value:
-**  	An unsigned integer which is one of the DKIM_SIGBH_* constants
+**  	An integer that is one of the DKIM_SIGBH_* constants
 **  	indicating the current state of "bh" evaluation of the signature.
 */
 
-extern unsigned int dkim_sig_getbh __P((DKIM_SIGINFO *sig));
+extern int dkim_sig_getbh __P((DKIM_SIGINFO *sig));
 
 /*
 **  DKIM_SIG_GETKEYSIZE -- retreive key size after verifying
@@ -940,7 +957,7 @@ extern DKIM_STAT dkim_sig_getcanons __P((DKIM_SIGINFO *sig, dkim_canon_t *hdr,
 **  	A DKIM_STAT_* constant.
 */
 
-extern DKIM_STAT dkim_set_user_context __P((DKIM *dkim, const void *ctx));
+extern DKIM_STAT dkim_set_user_context __P((DKIM *dkim, void *ctx));
 
 /*
 **  DKIM_GET_USER_CONTEXT -- retrieve DKIM handle user context
@@ -952,7 +969,7 @@ extern DKIM_STAT dkim_set_user_context __P((DKIM *dkim, const void *ctx));
 **  	User context pointer.
 */
 
-extern const void *dkim_get_user_context __P((DKIM *dkim));
+extern void *dkim_get_user_context __P((DKIM *dkim));
 
 /*
 **  DKIM_GET_MSGDATE -- retrieve value extracted from the Date: header
@@ -1015,7 +1032,7 @@ extern u_char *dkim_getuser __P((DKIM *dkim));
 **  	or NULL if none.
 */
 
-extern unsigned char *dkim_get_signer __P((DKIM *dkim));
+extern const unsigned char *dkim_get_signer __P((DKIM *dkim));
 
 /*
 **  DKIM_SET_SIGNER -- set DKIM signature's signer
@@ -1214,6 +1231,23 @@ extern const char *dkim_sig_geterrorstr __P((DKIM_SIGERROR sigerr));
 
 extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 
+/*
+**  DKIM_POLICY_GETQUERIES -- retrieve the queries needed to conduct an ADSP
+**                            evaluation
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	qi -- DKIM_QUERYINFO handle array (returned)
+**  	nqi -- number of entries in the "qi" array
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_policy_getqueries __P((DKIM *dkim,
+                                             DKIM_QUERYINFO ***qi,
+                                             unsigned int *nqi));
+
 /*
 **  DKIM_POLICY_STATE_NEW -- initialize and return a DKIM policy state handle
 **
@@ -1463,6 +1497,7 @@ extern DKIM_STAT dkim_set_margin __P((DKIM *dkim, int value));
 
 /*
 **  DKIM_GET_REPUTATION -- query reputation service about a signature
+**                         (OBSOLETE; moved to libdkimrep)
 **
 **  Parameters:
 **  	dkim -- DKIM handle
@@ -1471,10 +1506,6 @@ extern DKIM_STAT dkim_set_margin __P((DKIM *dkim, int value));
 **  	rep -- integer reputation (returned)
 **
 **  Return value:
-**  	DKIM_STAT_OK -- "rep" now contains a reputation
-**  	DKIM_STAT_NOKEY -- no reputation data available
-**  	DKIM_STAT_CANTVRFY -- data retrieval error of some kind
-**  	DKIM_STAT_INTERNAL -- internal error of some kind
 **  	DKIM_STAT_NOTIMPLEMENT -- not implemented
 */
 
@@ -1529,8 +1560,9 @@ extern unsigned long dkim_ssl_version __P((void));
 #define DKIM_FEATURE_DNSSEC		6
 #define DKIM_FEATURE_RESIGN		7
 #define DKIM_FEATURE_ATPS		8
+#define DKIM_FEATURE_XTAGS		9
 
-#define	DKIM_FEATURE_MAX		8
+#define	DKIM_FEATURE_MAX		9
 
 extern _Bool dkim_libfeature __P((DKIM_LIB *lib, u_int fc));
 
@@ -1545,7 +1577,7 @@ extern _Bool dkim_libfeature __P((DKIM_LIB *lib, u_int fc));
 **  	Library version, i.e. value of the OPENDKIM_LIB_VERSION macro.
 */
 
-extern unsigned long dkim_libversion __P((void));
+extern uint32_t dkim_libversion __P((void));
 
 /*
 **  DKIM_GET_SIGSUBSTRING -- retrieve a minimal signature substring for
@@ -1784,6 +1816,20 @@ extern void dkim_dns_set_query_waitreply __P((DKIM_LIB *,
                                                       size_t *, int *,
                                                       int *)));
 
+/*
+**  DKIM_ADD_XTAG -- add an extension tag/value
+**
+**  Parameters:
+**  	dkim -- DKIM signing handle to extend
+**  	tag -- name of tag to add
+**  	value -- value to include
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_add_xtag __P((DKIM *, const char *, const char *));
+
 /*
 **  DKIM_ATPS_CHECK -- check for Authorized Third Party Signing
 **
@@ -1800,6 +1846,31 @@ extern void dkim_dns_set_query_waitreply __P((DKIM_LIB *,
 extern DKIM_STAT dkim_atps_check __P((DKIM *, DKIM_SIGINFO *,
                                       struct timeval *, dkim_atps_t *res));
 
+/*
+**  DKIM_QI_GETNAME -- retrieve the DNS name from a DKIM_QUERYINFO object
+**
+**  Parameters:
+**  	query -- DKIM_QUERYINFO handle
+**
+**  Return value:
+**  	A pointer to a NULL-terminated string indicating the name to be
+**  	queried, or NULL on error.
+*/
+
+extern const char *dkim_qi_getname __P((DKIM_QUERYINFO *));
+
+/*
+**  DKIM_QI_GETTYPE -- retrieve the DNS RR type from a DKIM_QUERYINFO object
+**
+**  Parameters:
+**  	query -- DKIM_QUERYINFO handle
+**
+**  Return value:
+**  	The DNS RR type to be queried, or -1 on error.
+*/
+
+extern int dkim_qi_gettype __P((DKIM_QUERYINFO *));
+
 /*
 **  DKIM_BASE32_ENCODE -- encode a string using base32
 **
