@@ -2,19 +2,23 @@
 **  Copyright (c) 2005-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.11 2010/05/04 04:43:04 cm-msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.12.10.1 2010/10/27 21:43:08 cm-msk Exp $";
 #endif /* !lint */
 
+#include "build-config.h"
+
 /* system includes */
 #include <sys/types.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
 #include <netinet/in.h>
-#include <stdbool.h>
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif /* HAVE_STDBOOL_H */
 #include <ctype.h>
 #include <assert.h>
 #include <string.h>
@@ -787,81 +791,59 @@ dkim_check_dns_reply(unsigned char *ansbuf, size_t anslen,
 	return (trunc ? 1 : 0);
 }
 
-#ifdef USE_UNBOUND
 /*
-**  DKIM_TIMESPEC_PAST -- return TRUE iff the time described by a timespec
-**                        structure has passed
+**  DKIM_MIN_TIMEVAL -- determine the timeout to apply before reaching
+**                      one of two timevals
 **
 **  Parameters:
-**  	ts -- timespec structure to evaluate
+**  	t1 -- first timeout (absolute)
+**  	t2 -- second timeout (absolute) (may be NULL)
+**  	t -- final timeout (relative)
+**  	which -- which of t1 and t2 hit first
 **
 **  Return value:
-**  	TRUE if "tv" refers to a time in the past, false otherwise.
+**  	None.
 */
 
-_Bool
-dkim_timespec_past(struct timespec *ts)
+void
+dkim_min_timeval(struct timeval *t1, struct timeval *t2, struct timeval *t,
+                 struct timeval **which)
 {
+	struct timeval *next;
 	struct timeval now;
 
-	assert(ts != NULL);
+	assert(t1 != NULL);
+	assert(t != NULL);
 
-	(void) gettimeofday(&now, NULL);
-
-	if (now.tv_sec > ts->tv_sec ||
-	    (now.tv_sec == ts->tv_sec && now.tv_usec * 1000 > ts->tv_nsec))
-		return TRUE;
+	if (t2 == NULL ||
+	    t2->tv_sec > t1->tv_sec ||
+	    (t2->tv_sec == t1->tv_sec && t2->tv_usec > t1->tv_usec))
+		next = t1;
 	else
-		return FALSE;
-}
-
-/*
-**  DKIM_WAIT_FD -- wait for a descriptor to become read-ready
-**
-**  Parameters:
-**  	fd -- descriptor of interest
-**  	until -- maximum wait time
-**
-**  Return value:
-**  	1 -- descriptor is ready
-**  	0 -- timeout
-**  	-1 -- error
-*/
-
-int
-dkim_wait_fd(int fd, struct timespec *until)
-{
-	fd_set fds;
-	struct timeval now;
-	struct timeval left;
-
-	assert(fd >= 0);
-	assert(until != NULL);
+		next = t2;
 
 	(void) gettimeofday(&now, NULL);
 
-	if (until->tv_sec < now.tv_sec ||
-	    (until->tv_sec == now.tv_sec &&
-	     until->tv_nsec < now.tv_usec * 1000))
+	if (next->tv_sec < now.tv_sec ||
+	    (next->tv_sec == now.tv_sec && next->tv_usec < now.tv_usec))
 	{
-		left.tv_sec = 0;
-		left.tv_usec = 0;
+		t->tv_sec = 0;
+		t->tv_usec = 0;
 	}
 	else
 	{
-		left.tv_sec = until->tv_sec - now.tv_sec;
-		left.tv_usec = until->tv_nsec / 1000 - now.tv_usec;
-
-		if (until->tv_nsec / 1000 < now.tv_usec)
+		t->tv_sec = next->tv_sec - now.tv_sec;
+		if (next->tv_usec < now.tv_usec)
 		{
-			left.tv_usec += 1000000;
-			left.tv_sec--;
+			t->tv_sec--;
+			t->tv_usec = next->tv_usec - now.tv_usec + 1000000;
+		}
+		else
+		{
+			t->tv_usec = next->tv_usec - now.tv_usec;
 		}
 	}
 
-	FD_ZERO(&fds);
-	FD_SET(fd, &fds);
-
-	return select(fd + 1, &fds, NULL, NULL, &left);
+	if (which != NULL)
+		*which = next;
 }
-#endif /* USE_UNBOUND */
