@@ -2,7 +2,7 @@
 **  Copyright (c) 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef _OPENDKIM_DNS_H_
@@ -23,6 +23,11 @@ static char opendkim_dns_h_id[] = "@(#)$Id: opendkim-dns.h,v 1.2 2010/08/30 22:0
 # include <rbl.h>
 #endif /* _FFR_RBL */
 
+#ifdef _FFR_DKIM_REPUTATION
+/* libdkimrep includes */
+# include <dkim-rep.h>
+#endif /* _FFR_DKIM_REPUTATION */
+
 #ifdef USE_UNBOUND
 /* libunbound includes */
 # include <unbound.h>
@@ -39,6 +44,9 @@ extern int dkimf_unbound_setup __P((DKIM_LIB *, struct dkimf_unbound *));
 # ifdef _FFR_RBL
 extern int dkimf_rbl_unbound_setup __P((RBL *, struct dkimf_unbound *));
 # endif /* _FFR_RBL */
+# ifdef _FFR_DKIM_REPUTATION
+extern int dkimf_rep_unbound_setup __P((DKIM_REP, struct dkimf_unbound *));
+# endif /* _FFR_DKIM_REPUTATION */
 #endif /* USE_UNBOUND */
 
 #ifdef USE_ARLIB
@@ -50,6 +58,9 @@ extern int dkimf_arlib_setup __P((DKIM_LIB *, AR_LIB));
 # ifdef _FFR_RBL
 extern int dkimf_rbl_arlib_setup __P((RBL *, AR_LIB));
 # endif /* _FFR_RBL */
+# ifdef _FFR_DKIM_REPUTATION
+extern int dkimf_rep_arlib_setup __P((DKIM_REP, AR_LIB));
+# endif /* _FFR_DKIM_REPUTATION */
 #endif /* USE_ARLIB */
 
 #endif /* _OPENDKIM_DNS_H_ */
