@@ -51,6 +51,9 @@ static const luaL_Reg dkimf_lua_lib_setup[] =
 	{ "db_check",		dkimf_xs_dbquery	},
 	{ "db_close",		dkimf_xs_dbclose	},
 	{ "db_open",		dkimf_xs_dbopen		},
+# ifdef _FFR_LUA_GLOBALS
+	{ "export",		dkimf_xs_export		},
+# endif /* _FFR_LUA_GLOBALS */
 	{ "get_clienthost",	dkimf_xs_clienthost	},
 	{ "get_clientip",	dkimf_xs_clientip	},
 	{ "get_dbhandle",	dkimf_xs_dbhandle	},
@@ -68,6 +71,9 @@ static const luaL_Reg dkimf_lua_lib_setup[] =
 	{ "sign",		dkimf_xs_requestsig	},
 	{ "use_ltag",		dkimf_xs_setpartial	},
 	{ "verify",		dkimf_xs_verify		},
+# ifdef _FFR_XTAGS
+	{ "xtag",		dkimf_xs_xtag		},
+# endif /* _FFR_XTAGS */
 	{ NULL,			NULL			}
 };
 
@@ -76,6 +82,9 @@ static const luaL_Reg dkimf_lua_lib_screen[] =
 	{ "db_check",		dkimf_xs_dbquery	},
 	{ "db_close",		dkimf_xs_dbclose	},
 	{ "db_open",		dkimf_xs_dbopen		},
+# ifdef _FFR_LUA_GLOBALS
+	{ "export",		dkimf_xs_export		},
+# endif /* _FFR_LUA_GLOBALS */
 	{ "get_dbhandle",	dkimf_xs_dbhandle	},
 	{ "get_fromdomain",	dkimf_xs_fromdomain	},
 	{ "get_header",		dkimf_xs_getheader	},
@@ -97,6 +106,9 @@ static const luaL_Reg dkimf_lua_lib_screen[] =
 # ifdef _FFR_STATSEXT
 static const luaL_Reg dkimf_lua_lib_stats[] =
 {
+#  ifdef _FFR_LUA_GLOBALS
+	{ "export",		dkimf_xs_export		},
+#  endif /* _FFR_LUA_GLOBALS */
 	{ "get_header",		dkimf_xs_getheader	},
 	{ "get_mtasymbol",	dkimf_xs_getsymval	},
 	{ "get_policy",		dkimf_xs_getpolicy	},
@@ -108,6 +120,11 @@ static const luaL_Reg dkimf_lua_lib_stats[] =
 	{ "get_sighandle",	dkimf_xs_getsighandle	},
 	{ "log",		dkimf_xs_log		},
 	{ "parse_field",	dkimf_xs_parsefield	},
+#  ifdef _FFR_LUA_GLOBALS
+#   ifdef _FFR_RBL
+	{ "rbl_check",		dkimf_xs_rblcheck	},
+#   endif /* _FFR_RBL */
+#  endif /* _FFR_LUA_GLOBALS */
 	{ "rcpt_count",		dkimf_xs_rcptcount	},
 	{ "sig_bhresult",	dkimf_xs_sigbhresult	},
 	{ "sig_bodylength",	dkimf_xs_bodylength	},
@@ -153,6 +170,9 @@ static const luaL_Reg dkimf_lua_lib_final[] =
 	{ "sig_getdomain",	dkimf_xs_getsigdomain	},
 	{ "sig_getidentity",	dkimf_xs_getsigidentity	},
 	{ "sig_result",		dkimf_xs_sigresult	},
+# ifdef _FFR_XTAGS
+	{ "xtag",		dkimf_xs_xtag		},
+# endif /* _FFR_XTAGS */
 	{ NULL,			NULL			}
 };
 #endif /* DKIMF_LUA_CONTEXT_HOOKS */
@@ -500,6 +520,11 @@ dkimf_lua_setup_hook(void *ctx, const char *script, size_t scriptlen,
 	lua_pushlightuserdata(l, ctx);
 	lua_setglobal(l, "ctx");
 
+# ifdef _FFR_LUA_GLOBALS
+	/* import other globals */
+	dkimf_import_globals(ctx, l);
+# endif /* _FFR_LUA_GLOBALS */
+
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
 	{
 	  case 0:
@@ -631,6 +656,11 @@ dkimf_lua_screen_hook(void *ctx, const char *script, size_t scriptlen,
 	lua_pushlightuserdata(l, ctx);
 	lua_setglobal(l, "ctx");
 
+# ifdef _FFR_LUA_GLOBALS
+	/* import other globals */
+	dkimf_import_globals(ctx, l);
+# endif /* _FFR_LUA_GLOBALS */
+
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
 	{
 	  case 0:
@@ -872,6 +902,11 @@ dkimf_lua_stats_hook(void *ctx, const char *script, size_t scriptlen,
 	lua_pushlightuserdata(l, ctx);
 	lua_setglobal(l, "ctx");
 
+# ifdef _FFR_LUA_GLOBALS
+	/* import other globals */
+	dkimf_import_globals(ctx, l);
+# endif /* _FFR_LUA_GLOBALS */
+
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
 	{
 	  case 0:
@@ -1113,6 +1148,11 @@ dkimf_lua_final_hook(void *ctx, const char *script, size_t scriptlen,
 	lua_pushlightuserdata(l, ctx);
 	lua_setglobal(l, "ctx");
 
+# ifdef _FFR_LUA_GLOBALS
+	/* import other globals */
+	dkimf_import_globals(ctx, l);
+# endif /* _FFR_LUA_GLOBALS */
+
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
 	{
 	  case 0:
